import 'dart:async';
import 'dart:math';

import 'package:equatable/equatable.dart';
import 'package:flutter/material.dart';
import 'package:hydrated_bloc/hydrated_bloc.dart';

import 'package:petitbac/data/fetch_data_helper.dart';
import 'package:petitbac/models/data/game_item.dart';
import 'package:petitbac/models/game.dart';
import 'package:petitbac/models/settings_game.dart';
import 'package:petitbac/models/settings_global.dart';

part 'game_state.dart';

class GameCubit extends HydratedCubit<GameState> {
  GameCubit()
      : super(GameState(
          currentGame: Game.createNull(),
        ));

  void updateState(Game game) {
    emit(GameState(
      currentGame: game,
    ));
  }

  void refresh() {
    final Game game = Game(
      gameSettings: state.currentGame.gameSettings,
      globalSettings: state.currentGame.globalSettings,
      items: state.currentGame.items,
      isRunning: state.currentGame.isRunning,
      isFinished: state.currentGame.isFinished,
      countdown: state.currentGame.countdown,
      position: state.currentGame.position,
    );
    // game.dump();

    updateState(game);
  }

  void startNewGame({
    required GameSettings gameSettings,
    required GlobalSettings globalSettings,
  }) {
    Game newGame = Game.createNew(
      gameSettings: gameSettings,
      globalSettings: globalSettings,
    );

    newGame.dump();

    updateState(newGame);
    startTimer();
  }

  void quitGame() {
    state.currentGame.isRunning = false;
    refresh();
  }

  void next() {
    if (state.currentGame.position < (state.currentGame.gameSettings.itemsCount - 1)) {
      state.currentGame.position++;
      startTimer();
    } else {
      state.currentGame.isFinished = true;
    }
    refresh();
  }

  void startTimer() {
    int timerValue = state.currentGame.gameSettings.timerValue;

    if (timerValue != 0) {
      state.currentGame.countdown = timerValue;

      const Duration interval = Duration(seconds: 1);
      Timer.periodic(
        interval,
        (Timer timer) {
          if (state.currentGame.countdown != 0) {
            state.currentGame.countdown = max(state.currentGame.countdown - 1, 0);
            refresh();
          }

          if (state.currentGame.countdown == 0) {
            timer.cancel();
            if (state.currentGame.position ==
                (state.currentGame.gameSettings.itemsCount - 1)) {
              state.currentGame.isFinished = true;
            }
          }

          refresh();
        },
      );
    } else {
      if (state.currentGame.position == (state.currentGame.gameSettings.itemsCount - 1)) {
        state.currentGame.isFinished = true;
      }
    }

    refresh();
  }

  void pickNewItem() {
    state.currentGame.items[state.currentGame.position] = FetchDataHelper().getRandomItem();
    refresh();
  }

  void pickNewCategory() {
    GameItem newItem = GameItem(
      letter: state.currentGame.letter,
      category: FetchDataHelper().getRandomItem().category,
    );

    state.currentGame.items[state.currentGame.position] = newItem;
    refresh();
  }

  void pickNewLetter() {
    GameItem newItem = GameItem(
      letter: FetchDataHelper().getRandomItem().letter,
      category: state.currentGame.category,
    );

    state.currentGame.items[state.currentGame.position] = newItem;
    refresh();
  }

  @override
  GameState? fromJson(Map<String, dynamic> json) {
    Game currentGame = json['currentGame'] as Game;

    return GameState(
      currentGame: currentGame,
    );
  }

  @override
  Map<String, dynamic>? toJson(GameState state) {
    return <String, dynamic>{
      'currentGame': state.currentGame.toJson(),
    };
  }
}
