import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:petitbac/cubit/game_cubit.dart';
import 'package:petitbac/models/game.dart';
import 'package:petitbac/ui/widgets/helpers/outlined_text_widget.dart';
import 'package:petitbac/utils/color_extensions.dart';

class GamePositionIndicator extends StatelessWidget {
  const GamePositionIndicator({super.key});

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<GameCubit, GameState>(
      builder: (BuildContext context, GameState gameState) {
        final Game currentGame = gameState.currentGame;

        final int currentPosition = currentGame.position + 1;
        final int maxPosition = currentGame.gameSettings.itemsCount;

        // Normalized [0..1] value
        final double barValue = currentPosition / maxPosition;

        const barHeight = 30.0;
        const Color baseColor = Colors.grey;
        const Color textColor = Color.fromARGB(255, 238, 238, 238);
        const Color outlineColor = Color.fromARGB(255, 200, 200, 200);

        return Stack(
          alignment: Alignment.center,
          children: [
            LinearProgressIndicator(
              value: barValue,
              color: baseColor,
              backgroundColor: baseColor.darken(),
              minHeight: barHeight,
              borderRadius: const BorderRadius.all(Radius.circular(10)),
            ),
            OutlinedText(
              text: '$currentPosition/$maxPosition',
              fontSize: barHeight * 0.7,
              textColor: textColor,
              outlineColor: outlineColor,
            ),
          ],
        );
      },
    );
  }
}
